window.Gleam.initSearch([{"doc":"glemcached","title":"glemcached","content":"[![Package Version](https://img.shields.io/hexpm/v/glemcached)](https://hex.pm/packages/glemcached)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glemcached/)\n\n# glemcached\n\n[Memcached](https://memcached.org) client in pure gleam! (except the TCP client, of course! That&#39;s [mug](https://github.com/lpil/mug)).\n\nMade with the goal of fully supporting the [Memcached protocol](https://github.com/memcached/memcached/blob/master/doc/protocol.txt)!\n\n## Features\n\n(help wanted!)\n\n- [X] Authentication\n- [X] Text Commands\n- [ ] SSL\n- [ ] Meta Commands\n- [ ] Connection pooling\n- [ ] Cluster support\n- [ ] First class actor support\n- [ ] JavaScript (!Browser) support with promises\n\nBinary commands are deprecated and hence will not be implemented.\n\n## Install\n\n```sh\ngleam add glemcached\n```\n\n## Example with text commands\n```gleam\nimport gleam/bit_array\nimport gleam/io\nimport glemcached.{connect, with_authentication, with_timeout}\nimport glemcached/text\n\npub fn main() {\n  let assert Ok(mem) =\n    glemcached.new(&quot;localhost&quot;, 11_211)\n    |&gt; with_timeout(500)\n    |&gt; with_authentication(&quot;user&quot;, &quot;pass&quot;)\n    |&gt; connect()\n\n  let assert Ok(Nil) = text.set(mem, &quot;foo&quot;, 123, 0, &lt;&lt;&quot;Hello&quot;&gt;&gt;)\n  let assert Ok(True) = text.append(mem, &quot;foo&quot;, 123, 0, &lt;&lt;&quot;, world!&quot;&gt;&gt;)\n  let assert Ok([value]) = text.get(mem, [&quot;foo&quot;])\n  let assert Ok(value) = bit_array.to_string(value.data)\n\n  // clean up!\n  let assert Ok(True) = text.delete(mem, &quot;foo&quot;)\n\n  io.println(value)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/glemcached&gt;.\n\n## Development\n\n```sh\ngleam run -m glemcached/examples/text # Run the example\ngleam test  # Run the tests\n```\n","url":"index.html"},{"doc":"glemcached/types","title":"CommandError","content":"pub type CommandError {\n  GeneralError\n  ClientError(String)\n  ServerError(String)\n}\n Memcached returns three types of errors:\n ERROR -&gt; invalid command\n CLIENT_ERROR &lt;error&gt;-&gt; some error in the usage of a command.\n   &lt;error&gt; is a human readable string\n SERVER_ERROR &lt;error&gt;-&gt; some error in the execution of a command.\n   &lt;error&gt; is a human readable string\nGeneralError\n\n\nClientError(String)\n\n\nServerError(String)\n\n\nSynonyms:\nglemcached/types.CommandError\nglemcached/types CommandError","url":"glemcached/types.html#CommandError"},{"doc":"glemcached/types","title":"Memcached","content":"pub type Memcached {\n  Memcached(socket: Socket, timeout: Int)\n}\n A record that holds the TCP socket between the client\n and the memcached instance, and also holds the timeout\n value which can be changed if desired. It defaults to\n whatever value was set with `with_timeout`, or 1000, if\n `with_timeout` wasn&#39;t called.\nMemcached(socket: Socket, timeout: Int)\n\n\nSynonyms:\nglemcached/types.Memcached\nglemcached/types Memcached","url":"glemcached/types.html#Memcached"},{"doc":"glemcached/types","title":"MemcachedError","content":"pub type MemcachedError {\n  SocketError(mug.Error)\n  CommandError(CommandError)\n  ParseError(ParseError)\n}\n General Error type for all errors returned by\n this library.\nSocketError(mug.Error)\n Something happened while trying to use the TCP socket\n\nCommandError(CommandError)\n The memcached command failed to execute\n\nParseError(ParseError)\n The data returned by memcached wasn&#39;t able to be parsed.\n This error is usually NEVER raised, and if it is raised, which\n means that there is a bug in the protocol implementation in\n either this libarary or the Memcached instance.\n (Usually it&#39;s the former, so any bug reports are highly appreciated!)\n\nSynonyms:\nglemcached/types.MemcachedError\nglemcached/types MemcachedError","url":"glemcached/types.html#MemcachedError"},{"doc":"glemcached/types","title":"ParseError","content":"pub type ParseError {\n  InvalidResponse\n  ValueParseError\n}\n ParseError is returned when there is an error in parsing the \n response returned by the Memcached server. If this error is\n returned, there either exists a bug in this libarary, or in the\n Memcached instance&#39;s implementation. This library is tested on\n the official Memcached instance, therefore, if you&#39;re using this\n library with the official instance, and a ParseError is raised,\n there is definitely a bug with the library. Please report it &lt;3\nInvalidResponse\n When the response starts with an unexpected keyword\n\nValueParseError\n When there&#39;s an error trying to parse VALUE ...\n\nSynonyms:\nglemcached/types.ParseError\nglemcached/types ParseError","url":"glemcached/types.html#ParseError"},{"doc":"glemcached/types","title":"glemcached/types","content":"","url":"glemcached/types.html"},{"doc":"glemcached/text","title":"CasReturnType","content":"pub type CasReturnType {\n  CasStored\n  CasExists\n  CasNotFound\n}\n The result returned by the `cas` command. Since it has three possible\n values, it is not possible to use a boolean to represent them.\nCasStored\n Data was stored successfully, i.e. the data was not modified since it was fetched with g(e|a)ts.\n\nCasExists\n Data was not stored, i.e. the data was modified since it was fetched with g(e|a)ts.\n This value is also returned if the `cas_unique` value is invalid, but the key exists.\n\nCasNotFound\n The key does not exist.\n\nSynonyms:\nglemcached/text.CasReturnType\nglemcached/text CasReturnType","url":"glemcached/text.html#CasReturnType"},{"doc":"glemcached/text","title":"add","content":"pub fn add(\n  mem: Memcached,\n  key key: String,\n  flags flags: Int,\n  exptime exptime: Int,\n  data data: BitArray,\n) -&gt; Result(Bool, MemcachedError)\n Sets this key in Memcached only if it doesn&#39;t already exist.\n Returns True if the value was set, and False if not (i.e. the key already exists).\n Errors are only returned if the command fails to run.\n\n # Example\n\n \n add(\n   mem,\n   key: &quot;foo&quot;,\n   flags: flag.bit_array_to_uint16(&lt;&lt;1234&gt;&gt;),\n   exptime: 100,\n   data: &lt;&lt;&quot;Hello, world!&quot;&gt;&gt;,\n )\n \nSynonyms:\nglemcached/text.add\nglemcached/text add","url":"glemcached/text.html#add"},{"doc":"glemcached/text","title":"append","content":"pub fn append(\n  mem: Memcached,\n  key key: String,\n  flags flags: Int,\n  exptime exptime: Int,\n  data data: BitArray,\n) -&gt; Result(Bool, MemcachedError)\n Adds this value to the end of the existing data.\n Returns True if the value was set, and False if not (i.e. the key does not exist).\n Errors are only returned if the command fails to run.\n\n # Example\n\n \n append(\n   mem,\n   key: &quot;foo&quot;,\n   flags: flag.bit_array_to_uint16(&lt;&lt;1234&gt;&gt;),\n   exptime: 100,\n   data: &lt;&lt;&quot;Hi!&quot;&gt;&gt;,\n )\n \nSynonyms:\nglemcached/text.append\nglemcached/text append","url":"glemcached/text.html#append"},{"doc":"glemcached/text","title":"cas","content":"pub fn cas(\n  mem: Memcached,\n  key key: String,\n  flags flags: Int,\n  exptime exptime: Int,\n  cas_unique cas_unique: Int,\n  data data: BitArray,\n) -&gt; Result(CasReturnType, MemcachedError)\n Sets this value only if it wasn&#39;t modified since the time it was fetched with a\n CAS (check and store) command like gets/gats. These commands return a unique CAS\n value, which must be passed as the `cas_unique` parameter to this function.\n\n # Example\n\n \n cas(\n   mem,\n   key: &quot;foo&quot;,\n   flags: flag.bit_array_to_uint16(&lt;&lt;1234&gt;&gt;),\n   exptime: 100,\n   data: &lt;&lt;&quot;Hi!&quot;&gt;&gt;,\n   cas_unique: 10\n )\n \nSynonyms:\nglemcached/text.cas\nglemcached/text cas","url":"glemcached/text.html#cas"},{"doc":"glemcached/text","title":"decr","content":"pub fn decr(\n  mem: Memcached,\n  key: String,\n  value: Int,\n) -&gt; Result(Option(Int), MemcachedError)\n Decrements the value of the key by `value`.\n The value is interpreted to be a **64-bit unsigned integer**, and any\n underflows are replaced with zero by Memcached.\n The Ok value is an Option, which will be None if `key` does not exist.\n If `key`&#39;s value is not an integer, a CommandError will be raised with\n the message: `&quot;cannot increment or decrement non-numeric value&quot;`\n\n # Example\n\n \n decr(\n   mem,\n   key: &quot;foo&quot;\n   value: 1\n )\n \nSynonyms:\nglemcached/text.decr\nglemcached/text decr","url":"glemcached/text.html#decr"},{"doc":"glemcached/text","title":"delete","content":"pub fn delete(\n  mem: Memcached,\n  key: String,\n) -&gt; Result(Bool, MemcachedError)\n Deletes the key.\n Returns True if the key existed, otherwise returns False.\n\n # Example\n\n \n gets(\n   mem,\n   keys: [&quot;foo&quot;, &quot;bar&quot;, &quot;baz&quot;],\n )\n \nSynonyms:\nglemcached/text.delete\nglemcached/text delete","url":"glemcached/text.html#delete"},{"doc":"glemcached/text","title":"gat","content":"pub fn gat(\n  mem: Memcached,\n  exptime: Int,\n  keys: List(String),\n) -&gt; Result(List(Value), MemcachedError)\n Get and Touch.\n Change the expiration time of all `keys` to `exptime` (in seconds or unix epoch if `exptime`\n is greater than 30 days (30 * 24 * 60 * 60 seconds)). An expiration time of 0 indicates that\n the key never expires, and an expiration time of a negative number indicates immediate\n expiration. Returns the values of each of the `keys` (similar to the `get` function).\n `exptime` is specified before `keys` to mimic the equivalent Memcached command&#39;s syntax.\n\n # Example\n\n \n gat(\n   mem,\n   exptime: 100\n   keys: [&quot;foo&quot;, &quot;bar&quot;, &quot;baz&quot;]\n )\n \nSynonyms:\nglemcached/text.gat\nglemcached/text gat","url":"glemcached/text.html#gat"},{"doc":"glemcached/text","title":"gats","content":"pub fn gats(\n  mem: Memcached,\n  exptime: Int,\n  keys: List(String),\n) -&gt; Result(List(ValueCas), MemcachedError)\n Get and Touch returning unique CAS values.\n Change the expiration time of all `keys` to `exptime` (in seconds or unix epoch if `exptime`\n is greater than 30 days (30 * 24 * 60 * 60 seconds)). An expiration time of 0 indicates that\n the key never expires, and an expiration time of a negative number indicates immediate\n expiration. Returns the values of each of the `keys` (similar to the `gets` command).\n `exptime` is specified before `keys` to mimic the equivalent Memcached command&#39;s syntax.\n\n # Example\n\n \n gats(\n   mem,\n   exptime: 100\n   keys: [&quot;foo&quot;, &quot;bar&quot;, &quot;baz&quot;]\n )\n \nSynonyms:\nglemcached/text.gats\nglemcached/text gats","url":"glemcached/text.html#gats"},{"doc":"glemcached/text","title":"get","content":"pub fn get(\n  mem: Memcached,\n  keys keys: List(String),\n) -&gt; Result(List(Value), MemcachedError)\n Gets the values and flags of these keys.\n If a value is not found, that key will not exist in the returned list.\n\n # Example\n\n \n get(\n   mem,\n   keys: [&quot;foo&quot;, &quot;bar&quot;, &quot;baz&quot;],\n )\n \nSynonyms:\nglemcached/text.get\nglemcached/text get","url":"glemcached/text.html#get"},{"doc":"glemcached/text","title":"gets","content":"pub fn gets(\n  mem: Memcached,\n  keys keys: List(String),\n) -&gt; Result(List(ValueCas), MemcachedError)\n Gets the values, flags, and unique CAS values of these keys.\n If a value is not found, that key will not exist in the returned list.\n See the `cas` function to learn more about CAS.\n\n # Example\n\n \n gets(\n   mem,\n   keys: [&quot;foo&quot;, &quot;bar&quot;, &quot;baz&quot;],\n )\n \nSynonyms:\nglemcached/text.gets\nglemcached/text gets","url":"glemcached/text.html#gets"},{"doc":"glemcached/text","title":"incr","content":"pub fn incr(\n  mem: Memcached,\n  key: String,\n  value: Int,\n) -&gt; Result(Option(Int), MemcachedError)\n Increments the value of the key by `value`.\n The value is interpreted to be a **64-bit unsigned integer**, and any\n overflows are replaced with zero by Memcached.\n The Ok value is an Option, which will be None if `key` does not exist.\n If `key`&#39;s value is not an integer, a CommandError will be raised with\n the message: `&quot;cannot increment or decrement non-numeric value&quot;`\n\n # Example\n\n \n incr(\n   mem,\n   key: &quot;foo&quot;,\n   value: 1\n )\n \nSynonyms:\nglemcached/text.incr\nglemcached/text incr","url":"glemcached/text.html#incr"},{"doc":"glemcached/text","title":"prepend","content":"pub fn prepend(\n  mem: Memcached,\n  key key: String,\n  flags flags: Int,\n  exptime exptime: Int,\n  data data: BitArray,\n) -&gt; Result(Bool, MemcachedError)\n Adds this value to the start of the existing data.\n Returns True if the value was set, and False if not (i.e. the key does not exist).\n Errors are only returned if the command fails to run.\n\n # Example\n\n \n prepend(\n   mem,\n   key: &quot;foo&quot;,\n   flags: flag.bit_array_to_uint16(&lt;&lt;1234&gt;&gt;),\n   exptime: 100,\n   data: &lt;&lt;&quot;Hi!&quot;&gt;&gt;,\n )\n \nSynonyms:\nglemcached/text.prepend\nglemcached/text prepend","url":"glemcached/text.html#prepend"},{"doc":"glemcached/text","title":"replace","content":"pub fn replace(\n  mem: Memcached,\n  key key: String,\n  flags flags: Int,\n  exptime exptime: Int,\n  data data: BitArray,\n) -&gt; Result(Bool, MemcachedError)\n Sets this key in Memcached only if it does already exist.\n Returns True if the value was set, and False if not (i.e. the key does not exist).\n Errors are only returned if the command fails to run.\n\n # Example\n\n \n replace(\n   mem,\n   key: &quot;foo&quot;,\n   flags: flag.bit_array_to_uint16(&lt;&lt;1234&gt;&gt;),\n   exptime: 100,\n   data: &lt;&lt;&quot;Hello, world!&quot;&gt;&gt;,\n )\n \nSynonyms:\nglemcached/text.replace\nglemcached/text replace","url":"glemcached/text.html#replace"},{"doc":"glemcached/text","title":"set","content":"pub fn set(\n  mem: Memcached,\n  key key: String,\n  flags flags: Int,\n  exptime exptime: Int,\n  data data: BitArray,\n) -&gt; Result(Nil, MemcachedError)\n Stores this data in memcached. Overrides existing data if any. \n\n # Example\n\n \n set(\n   mem,\n   key: &quot;foo&quot;,\n   flags: flag.bit_array_to_uint16(&lt;&lt;1234&gt;&gt;),\n   exptime: 100,\n   data: &lt;&lt;&quot;Hello, world!&quot;&gt;&gt;,\n )\n |&gt; should.be_ok()\n |&gt; should.equal(Nil)\n \nSynonyms:\nglemcached/text.set\nglemcached/text set","url":"glemcached/text.html#set"},{"doc":"glemcached/text","title":"touch","content":"pub fn touch(\n  mem: Memcached,\n  key: String,\n  exptime: Int,\n) -&gt; Result(Bool, MemcachedError)\n Change the expiration time of `key` to `exptime` (in seconds or unix epoch if `exptime` is\n greater than 30 days (30 * 24 * 60 * 60 seconds)). An expiration time of 0 indicates that\n the key never expires, and an expiration time of a negative number indicates immediate\n expiration.\n\n # Example\n\n \n touch(\n   mem,\n   key: &quot;foo&quot;\n   exptime: 100\n )\n \nSynonyms:\nglemcached/text.touch\nglemcached/text touch","url":"glemcached/text.html#touch"},{"doc":"glemcached/text","title":"glemcached/text","content":"","url":"glemcached/text.html"},{"doc":"glemcached","title":"ConnectionError","content":"pub type ConnectionError {\n  ConnectError(mug.Error)\n  AuthenticationFailure\n}\n Error returned while connecting to the Memcached instance\nConnectError(mug.Error)\n Something went wrong trying to establish a connection\n\nAuthenticationFailure\n The (maybe) provided credentials are wrong\n\nSynonyms:\nglemcached.ConnectionError\nglemcached ConnectionError","url":"glemcached.html#ConnectionError"},{"doc":"glemcached","title":"ConnectionOptions","content":"pub type ConnectionOptions {\n  ConnectionOptions(\n    host: String,\n    port: Int,\n    timeout: Int,\n    username: String,\n    password: String,\n  )\n}\n This record holds the options used to connect (and authenticate)\n to the memcached instance.\nConnectionOptions(\n  host: String,\n  port: Int,\n  timeout: Int,\n  username: String,\n  password: String,\n)\n\n\nSynonyms:\nglemcached.ConnectionOptions\nglemcached ConnectionOptions","url":"glemcached.html#ConnectionOptions"},{"doc":"glemcached","title":"close","content":"pub fn close(mem: Memcached) -&gt; Result(Nil, Error)\n Close the connection\nSynonyms:\nglemcached.close\nglemcached close","url":"glemcached.html#close"},{"doc":"glemcached","title":"connect","content":"pub fn connect(\n  opts: ConnectionOptions,\n) -&gt; Result(Memcached, ConnectionError)\n Connect to a Memcached instance. This function creates a new TCP\n connection to host:port using mug. The connection times out after\n `timeout` millis.\n\n The returned Result has an Ok value of `Memcached`, which holds the\n connection socket and the command timeout value (default `timeout`),\n which can be changed with the `with_timeout` function. The Error\n value of the result is the mug.Error returned while creating the\n connection or checking for authentication.\n\n To check for authentication, the client sends an empty request to\n the instance, which will respond with `CLIENT_ERROR unauthorized`\n if authentication is required. Then, the client will send the\n credentials using a SET command. It sets the key `auth` with an\n expiration of `-1`.\n\n # Example\n\n \n let mem = new(host, port)\n |&gt; with_timeout(500)\n |&gt; with_authentication(&quot;user&quot;, &quot;pass&quot;)\n |&gt; connect()\n \nSynonyms:\nglemcached.connect\nglemcached connect","url":"glemcached.html#connect"},{"doc":"glemcached","title":"new","content":"pub fn new(host: String, port: Int) -&gt; ConnectionOptions\n Create a new `ConnectionOptions`\nSynonyms:\nglemcached.new\nglemcached new","url":"glemcached.html#new"},{"doc":"glemcached","title":"with_authentication","content":"pub fn with_authentication(\n  conn: ConnectionOptions,\n  username username: String,\n  password password: String,\n) -&gt; ConnectionOptions\n Set the username and password fields of `ConnectionOptions`\nSynonyms:\nglemcached.with_authentication\nglemcached with_authentication","url":"glemcached.html#with_authentication"},{"doc":"glemcached","title":"with_timeout","content":"pub fn with_timeout(\n  conn: ConnectionOptions,\n  timeout timeout: Int,\n) -&gt; ConnectionOptions\n Set the `ConnectionOptions`&#39; timeout value\nSynonyms:\nglemcached.with_timeout\nglemcached with_timeout","url":"glemcached.html#with_timeout"},{"doc":"glemcached","title":"glemcached","content":"","url":"glemcached.html"},{"doc":"glemcached/flag","title":"bitarray_to_uint16","content":"pub fn bitarray_to_uint16(ba: BitArray) -&gt; Int\n\nSynonyms:\nglemcached/flag.bitarray_to_uint16\nglemcached/flag bitarray_to_uint16","url":"glemcached/flag.html#bitarray_to_uint16"},{"doc":"glemcached/flag","title":"uint16_to_bitarray","content":"pub fn uint16_to_bitarray(num: Int) -&gt; BitArray\n\nSynonyms:\nglemcached/flag.uint16_to_bitarray\nglemcached/flag uint16_to_bitarray","url":"glemcached/flag.html#uint16_to_bitarray"},{"doc":"glemcached/flag","title":"glemcached/flag","content":"","url":"glemcached/flag.html"}]);